package adapterclient

import (
	"encoding/json"
	"fmt"
	"log"

	"github.com/pivotal-cf/on-demand-services-sdk/bosh"
	"github.com/pivotal-cf/on-demand-services-sdk/serviceadapter"
)

func (a *Adapter) CreateBinding(bindingID string, deploymentTopology bosh.BoshVMs, manifest []byte, requestParams map[string]interface{}, logger *log.Logger) (serviceadapter.Binding, error) {
	errs := func(err error) (serviceadapter.Binding, error) {
		return serviceadapter.Binding{}, err
	}

	serialisedBoshVMs, err := json.Marshal(deploymentTopology)
	if err != nil {
		return errs(err)
	}
	serialisedRequestParams, err := json.Marshal(requestParams)
	if err != nil {
		return errs(err)
	}

	stdout, stderr, exitcode, err := a.CommandRunner.Run(a.ExternalBinPath, "create-binding", bindingID, string(serialisedBoshVMs), string(manifest), string(serialisedRequestParams))

	e := newErrorFactory(a.ExternalBinPath, stdout, stderr)
	if err != nil {
		return errs(e.adapterError(err))
	}

	exitError := e.exitCodeError(err, *exitcode)
	switch *exitcode {
	case SuccessExitCode:
		break
	case serviceadapter.NotImplementedExitCode:
		return errs(fmt.Errorf("service adapter has not implemented create-binding command: %s", exitError))
	case serviceadapter.AppGuidNotProvidedErrorExitCode:
		return errs(serviceadapter.NewAppGuidNotProvidedError(exitError))
	case serviceadapter.BindingAlreadyExistsErrorExitCode:
		return errs(serviceadapter.NewBindingAlreadyExistsError(exitError))
	default:
		return errs(exitError)
	}

	logger.Printf("service adapter ran create-binding successfully, stderr logs: %s", string(stderr))

	var adapterBinding serviceadapter.Binding
	if err := json.Unmarshal(stdout, &adapterBinding); err != nil {
		return errs(err)
	}

	return adapterBinding, nil
}
