package boshclient_test

import (
	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
	"github.com/pivotal-cf/on-demand-service-broker/mockbosh"
)

var _ = Describe("running errands", func() {
	const (
		deploymentName = "deploymentName"
		errandName     = "errandName"
		contextID      = "some-context-id"
	)

	Context("successfully", func() {
		It("invokes BOSH to queue up an errand", func() {
			taskID := 5
			director.VerifyAndMock(
				mockbosh.Errand(deploymentName, errandName).WithContextID(contextID).RedirectsToTask(taskID),
			)

			actualTaskID, actualErr := c.RunErrand(deploymentName, errandName, contextID, logger)
			Expect(actualTaskID).To(Equal(taskID))
			Expect(actualErr).NotTo(HaveOccurred())
		})
	})

	Context("has an error", func() {
		It("invokes BOSH to queue up an errand", func() {
			director.VerifyAndMock(
				mockbosh.Errand(deploymentName, errandName).WithAnyContextID().Fails("because reasons"),
			)

			_, actualErr := c.RunErrand(deploymentName, errandName, contextID, logger)
			Expect(actualErr).To(HaveOccurred())
		})
	})
})
