package boshclient

import "encoding/json"

type BoshTasks []BoshTask

func (t BoshTasks) IncompleteTasks() BoshTasks {
	return t.findTasksInStates(BoshTaskQueued, BoshTaskProcessing, BoshTaskCancelling)
}

func (t BoshTasks) FailedTasks() BoshTasks {
	return t.findTasksInStates(BoshTaskError, BoshTaskCancelled, BoshTaskTimeout)
}

func (t BoshTasks) DoneTasks() BoshTasks {
	return t.findTasksInStates(BoshTaskDone)
}

func (t BoshTasks) findTasksInStates(states ...string) BoshTasks {
	found := BoshTasks{}
	for _, task := range t {
		for _, expectedState := range states {
			if task.State == expectedState {
				found = append(found, task)
			}
		}
	}
	return found
}

func (t BoshTasks) ToLog() string {
	output, _ := json.Marshal(t)
	// json.Marshal will only return an error when the struct contains an
	// unsupported type. We know our struct does not, so we don't handle this
	// error.
	return string(output)
}
