package broker

import (
	"context"
	"fmt"
	"log"

	"github.com/pivotal-cf/brokerapi"
)

func (b *Broker) Instances(logger *log.Logger) ([]string, error) {
	instanceIDs, err := b.cfClient.GetInstancesOfServiceOffering(b.serviceOffering.ID, logger)
	if err != nil {
		logger.Printf("error listing instances: %s", err)
		return nil, err
	}

	return instanceIDs, nil
}

func (b *Broker) validatePlanQuota(ctx context.Context, serviceID string, plan Plan, logger *log.Logger) DisplayableError {
	if plan.Quotas == nil {
		return NilError
	}

	count, err := b.cfClient.CountInstancesOfPlan(serviceID, plan.ID, logger)
	if err != nil {
		return NewGenericError(ctx, fmt.Errorf("error counting instances of plan: %s", err))
	}

	if count >= plan.Quotas.ServiceInstanceLimit {
		return NewDisplayableError(brokerapi.ErrPlanQuotaExceeded, fmt.Errorf("plan quota exceeded for plan ID %s", plan.ID))
	}

	return NilError
}
