package brokercontext

import "context"

type correlationIDType int

const (
	operationKey   correlationIDType = iota
	requestIDKey   correlationIDType = iota
	serviceNameKey correlationIDType = iota
	instanceIDKey  correlationIDType = iota
	boshTaskIDKey  correlationIDType = iota
)

func New(ctx context.Context, operation, requestID, serviceName, instanceID string) context.Context {
	ctx = WithOperation(ctx, operation)
	ctx = WithReqID(ctx, requestID)
	ctx = WithServiceName(ctx, serviceName)
	ctx = WithInstanceID(ctx, instanceID)
	return ctx
}

func WithOperation(ctx context.Context, operation string) context.Context {
	return context.WithValue(ctx, operationKey, operation)
}

func GetOperation(ctx context.Context) string {
	operation, _ := ctx.Value(operationKey).(string)
	return operation
}

func WithReqID(ctx context.Context, reqID string) context.Context {
	return context.WithValue(ctx, requestIDKey, reqID)
}

func GetReqID(ctx context.Context) string {
	reqID, _ := ctx.Value(requestIDKey).(string)
	return reqID
}

func WithServiceName(ctx context.Context, serviceName string) context.Context {
	return context.WithValue(ctx, serviceNameKey, serviceName)
}

func GetServiceName(ctx context.Context) string {
	serviceName, _ := ctx.Value(serviceNameKey).(string)
	return serviceName
}

func WithInstanceID(ctx context.Context, instanceID string) context.Context {
	return context.WithValue(ctx, instanceIDKey, instanceID)
}

func GetInstanceID(ctx context.Context) string {
	instanceID, _ := ctx.Value(instanceIDKey).(string)
	return instanceID
}

func WithBoshTaskID(ctx context.Context, boshTaskID int) context.Context {
	return context.WithValue(ctx, boshTaskIDKey, boshTaskID)
}

func GetBoshTaskID(ctx context.Context) int {
	boshTaskID, _ := ctx.Value(boshTaskIDKey).(int)
	return boshTaskID
}
