package main

import (
	"flag"
	"fmt"
	"log"
	"net/http"
	"os"
	"time"

	"github.com/craigfurman/herottp"
	"github.com/pivotal-cf/on-demand-service-broker/loggerfactory"
)

func main() {
	brokerUsername := flag.String("brokerUsername", "", "username for the broker")
	brokerPassword := flag.String("brokerPassword", "", "password for the broker")
	brokerPort := flag.String("brokerPort", "", "port of the broker")
	timeout := flag.Int("timeout", 120, "timeout in seconds for the post-start check")
	flag.Parse()

	prefix := "[broker-post-start] "
	logFlags := log.Ldate | log.Ltime | log.Lmicroseconds | log.LUTC
	loggerFactory := loggerfactory.New(os.Stdout, prefix, logFlags)

	logger := loggerFactory.NewLoggerWithoutRequestID()
	logger.Println("Starting broker post-start check, waiting for broker to start serving catalog.")

	brokerCatalogURL := fmt.Sprintf("http://localhost:%s/v2/catalog", *brokerPort)
	request, err := http.NewRequest("GET", brokerCatalogURL, nil)
	if err != nil {
		logger.Printf("error creating request: %s\n", err)
		os.Exit(1)
	}
	request.SetBasicAuth(*brokerUsername, *brokerPassword)

	client := herottp.New(herottp.Config{Timeout: 30 * time.Second})

	go func() {
		for {
			response, err := client.Do(request)
			if err != nil {
				logger.Printf("error performing request: %s", err)
			} else if response.StatusCode != http.StatusOK {
				logger.Printf("expected status 200, was %d, from %s\n", response.StatusCode, brokerCatalogURL)
			} else {
				logger.Println("Broker post-start check successful")
				os.Exit(0)
			}
			time.Sleep(1 * time.Second)
		}
	}()

	<-time.After(time.Duration(*timeout) * time.Second)

	logger.Println("Broker post-start check failed")
	os.Exit(1)
}
