package loggerfactory

import (
	"context"
	"fmt"
	"io"
	"log"

	"github.com/pborman/uuid"
	"github.com/pivotal-cf/on-demand-service-broker/brokercontext"
)

type LoggerFactory struct {
	out    io.Writer
	prefix string
	flag   int
}

func New(out io.Writer, prefix string, flag int) *LoggerFactory {
	return &LoggerFactory{out: out, prefix: prefix, flag: flag}
}

func (l *LoggerFactory) NewLoggerWithContext(ctx context.Context) *log.Logger {
	if brokercontext.GetReqID(ctx) == "" {
		return l.NewLoggerWithoutRequestID()
	}

	prefix := fmt.Sprintf("%s[%s] ", l.prefix, brokercontext.GetReqID(ctx))
	return log.New(l.out, prefix, l.flag)
}

func (l *LoggerFactory) NewLogger() *log.Logger {
	prefix := fmt.Sprintf("%s[%s] ", l.prefix, uuid.New())
	return log.New(l.out, prefix, l.flag)
}

func (l *LoggerFactory) NewLoggerWithoutRequestID() *log.Logger {
	return log.New(l.out, l.prefix, l.flag)
}
