package loggerfactory_test

import (
	"bytes"
	"context"
	"log"

	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
	"github.com/pivotal-cf/on-demand-service-broker/brokercontext"
	"github.com/pivotal-cf/on-demand-service-broker/loggerfactory"
)

var _ = Describe("LoggerFactory", func() {
	Context("when request ID is present in context", func() {
		It("can return a logger that logs messages with request ID", func() {
			logs := &bytes.Buffer{}
			requestID := "some-uuid"
			ctx := context.Background()
			ctx = brokercontext.WithReqID(ctx, requestID)
			factory := loggerfactory.New(logs, "[prefix] ", log.LstdFlags)

			logger := factory.NewLoggerWithContext(ctx)
			logger.Println("ohai")

			Expect(logs.String()).To(MatchRegexp(`\[prefix\] \[some-uuid\] \d{4}/\d{2}/\d{2} \d{2}:\d{2}:\d{2} ohai`))
		})

		It("can get the request ID from the context", func() {
			requestID := "some-uuid"
			ctx := context.Background()
			ctx = brokercontext.WithReqID(ctx, requestID)

			actualID := brokercontext.GetReqID(ctx)

			Expect(actualID).To(Equal(requestID))
		})
	})

	Context("when request ID not present in context", func() {
		It("can return a logger that logs messages without request ID", func() {
			logs := &bytes.Buffer{}
			ctx := context.Background()
			factory := loggerfactory.New(logs, "[prefix] ", log.LstdFlags)

			logger := factory.NewLoggerWithContext(ctx)
			logger.Println("ohai")

			Expect(logs.String()).To(MatchRegexp(`\[prefix\] \d{4}/\d{2}/\d{2} \d{2}:\d{2}:\d{2} ohai`))
		})

		It("cannot get the request ID from the context", func() {
			ctx := context.Background()

			actualID := brokercontext.GetReqID(ctx)

			Expect(actualID).To(BeEmpty())
		})
	})

	It("can return a logger that logs messages without request ID", func() {
		logs := &bytes.Buffer{}
		factory := loggerfactory.New(logs, "[prefix] ", log.LstdFlags)

		logger := factory.NewLoggerWithoutRequestID()
		logger.Println("ohai")

		Expect(logs.String()).To(MatchRegexp(`\[prefix\] \d{4}/\d{2}/\d{2} \d{2}:\d{2}:\d{2} ohai`))
	})
})
